import { Metadata } from "revolt-api";
import { API, Client } from "..";
/**
 * Uploaded File
 */
export declare class File {
    #private;
    /**
     * File Id
     */
    readonly id: string;
    /**
     * File bucket
     */
    readonly tag: string;
    /**
     * Original filename
     */
    readonly filename: string;
    /**
     * Parsed metadata of the file
     */
    readonly metadata: Metadata;
    /**
     * Raw content type of this file
     */
    readonly contentType: string;
    /**
     * Size of the file (in bytes)
     */
    readonly size: number;
    /**
     * Construct File
     * @param client Client
     * @param file File
     */
    constructor(client: Client, file: API.File);
    /**
     * Direct URL to the file
     */
    get url(): string;
    /**
     * Download URL for the file
     */
    get downloadURL(): string;
    /**
     * Human readable file size
     */
    get humanReadableSize(): string;
    /**
     * Whether this file should be spoilered
     */
    get isSpoiler(): boolean;
    /**
     * Creates a URL to a given file with given options.
     * @param options Optional query parameters to modify object
     * @param allowAnimation Returns GIF if applicable, no operations occur on image
     * @returns Generated URL or nothing
     */
    createFileURL(options?: {
        max_side?: number;
        size?: number;
        width?: number;
        height?: number;
    }, allowAnimation?: boolean): string | undefined;
}
//# sourceMappingURL=File.d.ts.map