"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Message_collection;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Message = void 0;
const ulid_1 = require("ulid");
/**
 * Message Class
 */
class Message {
    /**
     * Construct Message
     * @param collection Collection
     * @param id Message Id
     */
    constructor(collection, id) {
        _Message_collection.set(this, void 0);
        __classPrivateFieldSet(this, _Message_collection, collection, "f");
        this.id = id;
    }
    /**
     * Time when this message was posted
     */
    get createdAt() {
        return new Date((0, ulid_1.decodeTime)(this.id));
    }
    /**
     * Absolute pathname to this message in the client
     */
    get path() {
        return `${this.channel.path}/${this.id}`;
    }
    /**
     * URL to this message
     */
    get url() {
        var _a;
        return ((_a = __classPrivateFieldGet(this, _Message_collection, "f").client.configuration) === null || _a === void 0 ? void 0 : _a.app) + this.path;
    }
    /**
     * Nonce value
     */
    get nonce() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).nonce;
    }
    /**
     * Id of channel this message was sent in
     */
    get channelId() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).channelId;
    }
    /**
     * Channel this message was sent in
     */
    get channel() {
        return __classPrivateFieldGet(this, _Message_collection, "f").client.channels.get(__classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).channelId);
    }
    /**
     * Server this message was sent in
     */
    get server() {
        return this.channel.server;
    }
    /**
     * Member this message was sent by
     */
    get member() {
        return __classPrivateFieldGet(this, _Message_collection, "f").client.serverMembers.getByKey({
            server: this.channel.serverId,
            user: this.authorId,
        });
    }
    /**
     * Id of user this message was sent by
     */
    get authorId() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).authorId;
    }
    /**
     * User this message was sent by
     */
    get author() {
        return __classPrivateFieldGet(this, _Message_collection, "f").client.users.get(__classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).authorId);
    }
    /**
     * Content
     */
    get content() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).content;
    }
    /**
     * System message content
     */
    get systemMessage() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).systemMessage;
    }
    /**
     * Attachments
     */
    get attachments() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).attachments;
    }
    /**
     * Time at which this message was edited
     */
    get editedAt() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).editedAt;
    }
    /**
     * Embeds
     */
    get embeds() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).embeds;
    }
    /**
     * IDs of users this message mentions
     */
    get mentionIds() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).mentionIds;
    }
    /**
     * IDs of messages this message replies to
     */
    get replyIds() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).replyIds;
    }
    /**
     * Reactions
     */
    get reactions() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).reactions;
    }
    /**
     * Interactions
     */
    get interactions() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).interactions;
    }
    /**
     * Masquerade
     */
    get masquerade() {
        return __classPrivateFieldGet(this, _Message_collection, "f").getUnderlyingObject(this.id).masquerade;
    }
    /**
     * Get the username for this message
     */
    get username() {
        var _a, _b, _c, _d, _e;
        return ((_d = (_b = (_a = this.masquerade) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : (_c = this.member) === null || _c === void 0 ? void 0 : _c.nickname) !== null && _d !== void 0 ? _d : (_e = this.author) === null || _e === void 0 ? void 0 : _e.username);
    }
    /**
     * Get the role colour for this message
     */
    get roleColour() {
        var _a, _b, _c;
        return (_b = (_a = this.masquerade) === null || _a === void 0 ? void 0 : _a.colour) !== null && _b !== void 0 ? _b : (_c = this.member) === null || _c === void 0 ? void 0 : _c.roleColour;
    }
    /**
     * Get the avatar URL for this message
     */
    get avatarURL() {
        var _a, _b, _c, _d;
        return ((_c = (_a = this.masqueradeAvatarURL) !== null && _a !== void 0 ? _a : (_b = this.member) === null || _b === void 0 ? void 0 : _b.avatarURL) !== null && _c !== void 0 ? _c : (_d = this.author) === null || _d === void 0 ? void 0 : _d.avatarURL);
    }
    /**
     * Get the animated avatar URL for this message
     */
    get animatedAvatarURL() {
        var _a, _b, _c;
        return ((_a = this.masqueradeAvatarURL) !== null && _a !== void 0 ? _a : (this.member
            ? (_b = this.member) === null || _b === void 0 ? void 0 : _b.animatedAvatarURL
            : (_c = this.author) === null || _c === void 0 ? void 0 : _c.animatedAvatarURL));
    }
    /**
     * Avatar URL from the masquerade
     */
    get masqueradeAvatarURL() {
        var _a;
        const avatar = (_a = this.masquerade) === null || _a === void 0 ? void 0 : _a.avatar;
        return avatar ? __classPrivateFieldGet(this, _Message_collection, "f").client.proxyFile(avatar) : undefined;
    }
    /**
     * Populated system message
     */
    get populatedSystemMessage() {
        const system = this.systemMessage;
        if (!system)
            return { type: "none" };
        const { type } = system;
        /**
         * Get user by their id
         * @param id Id
         * @returns User
         */
        const get = (id) => __classPrivateFieldGet(this, _Message_collection, "f").client.users.get(id);
        switch (system.type) {
            case "text":
                return system;
            case "user_added":
                return { type, user: get(system.id), by: get(system.by) };
            case "user_remove":
                return { type, user: get(system.id), by: get(system.by) };
            case "user_joined":
                return { type, user: get(system.id) };
            case "user_left":
                return { type, user: get(system.id) };
            case "user_kicked":
                return { type, user: get(system.id) };
            case "user_banned":
                return { type, user: get(system.id) };
            case "channel_renamed":
                return { type, name: system.name, by: get(system.by) };
            case "channel_description_changed":
                return { type, by: get(system.by) };
            case "channel_icon_changed":
                return { type, by: get(system.by) };
            case "channel_ownership_changed":
                return { type, from: get(system.from), to: get(system.to) };
        }
    }
    /**
     * Edit a message
     * @param data Message edit route data
     */
    edit(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield __classPrivateFieldGet(this, _Message_collection, "f").client.api.patch(`/channels/${this.channelId}/messages/${this.id}`, data);
        });
    }
    /**
     * Delete a message
     */
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield __classPrivateFieldGet(this, _Message_collection, "f").client.api.delete(`/channels/${this.channelId}/messages/${this.id}`);
        });
    }
    /**
     * Acknowledge this message as read
     */
    ack() {
        var _a;
        (_a = this.channel) === null || _a === void 0 ? void 0 : _a.ack(this);
    }
    /**
     * Reply to Message
     */
    reply(data, mention = true) {
        var _a;
        const obj = typeof data === "string" ? { content: data } : data;
        return (_a = this.channel) === null || _a === void 0 ? void 0 : _a.sendMessage(Object.assign(Object.assign({}, obj), { replies: [{ id: this.id, mention }] }));
    }
    /**
     * Clear all reactions from this message
     */
    clearReactions() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield __classPrivateFieldGet(this, _Message_collection, "f").client.api.delete(`/channels/${this.channelId}/messages/${this.id}/reactions`);
        });
    }
    /**
     * React to a message
     * @param emoji Unicode or emoji ID
     */
    react(emoji) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield __classPrivateFieldGet(this, _Message_collection, "f").client.api.put(`/channels/${this.channelId}/messages/${this.id}/reactions/${emoji}`);
        });
    }
    /**
     * Un-react from a message
     * @param emoji Unicode or emoji ID
     */
    unreact(emoji) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield __classPrivateFieldGet(this, _Message_collection, "f").client.api.delete(`/channels/${this.channelId}/messages/${this.id}/reactions/${emoji}`);
        });
    }
}
exports.Message = Message;
_Message_collection = new WeakMap();
