import { API, Client, File } from "..";
import { ServerFlags } from "../hydration/server";
/**
 * Public Channel Invite
 */
export declare abstract class PublicChannelInvite {
    protected client?: Client;
    readonly type: API.Invite["type"] | "None";
    /**
     * Construct Channel Invite
     * @param client Client
     * @param type Type
     */
    constructor(client?: Client, type?: API.Invite["type"] | "None");
    /**
     * Create an Invite from an API Invite Response
     * @param client Client
     * @param invite Data
     * @returns Invite
     */
    static from(client: Client, invite: API.InviteResponse): PublicChannelInvite;
}
/**
 * Public invite of unknown type
 */
export declare class UnknownPublicInvite extends PublicChannelInvite {
}
/**
 * Public Server Invite
 */
export declare class ServerPublicInvite extends PublicChannelInvite {
    readonly code: string;
    readonly userName: string;
    readonly userAvatar?: File;
    readonly serverId: string;
    readonly serverName: string;
    readonly serverIcon?: File;
    readonly serverBanner?: File;
    readonly serverFlags: ServerFlags;
    readonly memberCount: number;
    readonly channelId: string;
    readonly channelName: string;
    readonly channelDescription?: string;
    /**
     * Construct Server Invite
     * @param client Client
     * @param invite Invite
     */
    constructor(client: Client, invite: API.InviteResponse & {
        type: "Server";
    });
    /**
     * Join the server
     */
    join(): Promise<import("./Server").Server>;
}
//# sourceMappingURL=PublicInvite.d.ts.map