import { API, Message } from "..";
import { HydratedMessage } from "../hydration";
import { ClassCollection } from ".";
export declare class MessageCollection extends ClassCollection<Message, HydratedMessage> {
    /**
     * Fetch message by Id
     * @param channelId Channel Id
     * @param messageId Message Id
     * @returns Message
     */
    fetch(channelId: string, messageId: string): Promise<Message>;
    /**
     * Get or create
     * @param id Id
     * @param data Data
     * @param isNew Whether this object is new
     */
    getOrCreate(id: string, data: API.Message, isNew?: boolean): Message;
    /**
     * Get or return partial
     * @param id Id
     */
    getOrPartial(id: string): Message | undefined;
    /**
     * Globally fetch messages
     * @requires Admin
     * @param query Message query
     */
    queryMessages(query: API.MessageQuery): Promise<{
        _id: string;
        nonce?: string | null | undefined;
        channel: string;
        author: string;
        content?: string | null | undefined;
        system?: {
            type: "text";
            content: string;
        } | {
            type: "user_added";
            id: string;
            by: string;
        } | {
            type: "user_remove";
            id: string;
            by: string;
        } | {
            type: "user_joined";
            id: string;
        } | {
            type: "user_left";
            id: string;
        } | {
            type: "user_kicked";
            id: string;
        } | {
            type: "user_banned";
            id: string;
        } | {
            type: "channel_renamed";
            name: string;
            by: string;
        } | {
            type: "channel_description_changed";
            by: string;
        } | {
            type: "channel_icon_changed";
            by: string;
        } | {
            type: "channel_ownership_changed";
            from: string;
            to: string;
        } | null | undefined;
        attachments?: {
            _id: string;
            tag: string;
            filename: string;
            metadata: {
                type: "File";
            } | {
                type: "Text";
            } | {
                type: "Image";
                width: number;
                height: number;
            } | {
                type: "Video";
                width: number;
                height: number;
            } | {
                type: "Audio";
            };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
        }[] | null | undefined;
        edited?: string | null | undefined;
        embeds?: ({
            type: "Website";
            url?: string | null | undefined;
            original_url?: string | null | undefined;
            special?: {
                type: "None";
            } | {
                type: "GIF";
            } | {
                type: "YouTube";
                id: string;
                timestamp?: string | null | undefined;
            } | {
                type: "Lightspeed";
                content_type: "Channel";
                id: string;
            } | {
                type: "Twitch";
                content_type: "Channel" | "Video" | "Clip";
                id: string;
            } | {
                type: "Spotify";
                content_type: string;
                id: string;
            } | {
                type: "Soundcloud";
            } | {
                type: "Bandcamp";
                content_type: "Album" | "Track";
                id: string;
            } | {
                type: "Streamable";
                id: string;
            } | null | undefined;
            title?: string | null | undefined;
            description?: string | null | undefined;
            image?: {
                url: string;
                width: number;
                height: number;
                size: "Large" | "Preview";
            } | null | undefined;
            video?: {
                url: string;
                width: number;
                height: number;
            } | null | undefined;
            site_name?: string | null | undefined;
            icon_url?: string | null | undefined;
            colour?: string | null | undefined;
        } | {
            type: "Image";
            url: string;
            width: number;
            height: number;
            size: "Large" | "Preview";
        } | {
            type: "Video";
            url: string;
            width: number;
            height: number;
        } | {
            type: "Text";
            icon_url?: string | null | undefined;
            url?: string | null | undefined;
            title?: string | null | undefined;
            description?: string | null | undefined;
            media?: {
                _id: string;
                tag: string;
                filename: string;
                metadata: {
                    type: "File";
                } | {
                    type: "Text";
                } | {
                    type: "Image";
                    width: number;
                    height: number;
                } | {
                    type: "Video";
                    width: number;
                    height: number;
                } | {
                    type: "Audio";
                };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
            } | null | undefined;
            colour?: string | null | undefined;
        } | {
            type: "None";
        })[] | null | undefined;
        mentions?: string[] | null | undefined;
        replies?: string[] | null | undefined;
        reactions?: {
            [key: string]: string[];
        } | undefined;
        interactions?: {
            reactions?: string[] | null | undefined;
            restrict_reactions?: boolean | undefined;
        } | undefined;
        masquerade?: {
            name?: string | null | undefined;
            avatar?: string | null | undefined;
            colour?: string | null | undefined;
        } | null | undefined;
    }[] | {
        messages: {
            _id: string;
            nonce?: string | null | undefined;
            channel: string;
            author: string;
            content?: string | null | undefined;
            system?: {
                type: "text";
                content: string;
            } | {
                type: "user_added";
                id: string;
                by: string;
            } | {
                type: "user_remove";
                id: string;
                by: string;
            } | {
                type: "user_joined";
                id: string;
            } | {
                type: "user_left";
                id: string;
            } | {
                type: "user_kicked";
                id: string;
            } | {
                type: "user_banned";
                id: string;
            } | {
                type: "channel_renamed";
                name: string;
                by: string;
            } | {
                type: "channel_description_changed";
                by: string;
            } | {
                type: "channel_icon_changed";
                by: string;
            } | {
                type: "channel_ownership_changed";
                from: string;
                to: string;
            } | null | undefined;
            attachments?: {
                _id: string;
                tag: string;
                filename: string;
                metadata: {
                    type: "File";
                } | {
                    type: "Text";
                } | {
                    type: "Image";
                    width: number;
                    height: number;
                } | {
                    type: "Video";
                    width: number;
                    height: number;
                } | {
                    type: "Audio";
                };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
            }[] | null | undefined;
            edited?: string | null | undefined;
            embeds?: ({
                type: "Website";
                url?: string | null | undefined;
                original_url?: string | null | undefined;
                special?: {
                    type: "None";
                } | {
                    type: "GIF";
                } | {
                    type: "YouTube";
                    id: string;
                    timestamp?: string | null | undefined;
                } | {
                    type: "Lightspeed";
                    content_type: "Channel";
                    id: string;
                } | {
                    type: "Twitch";
                    content_type: "Channel" | "Video" | "Clip";
                    id: string;
                } | {
                    type: "Spotify";
                    content_type: string;
                    id: string;
                } | {
                    type: "Soundcloud";
                } | {
                    type: "Bandcamp";
                    content_type: "Album" | "Track";
                    id: string;
                } | {
                    type: "Streamable";
                    id: string;
                } | null | undefined;
                title?: string | null | undefined;
                description?: string | null | undefined;
                image?: {
                    url: string;
                    width: number;
                    height: number;
                    size: "Large" | "Preview";
                } | null | undefined;
                video?: {
                    url: string;
                    width: number;
                    height: number;
                } | null | undefined;
                site_name?: string | null | undefined;
                icon_url?: string | null | undefined;
                colour?: string | null | undefined;
            } | {
                type: "Image";
                url: string;
                width: number;
                height: number;
                size: "Large" | "Preview";
            } | {
                type: "Video";
                url: string;
                width: number;
                height: number;
            } | {
                type: "Text";
                icon_url?: string | null | undefined;
                url?: string | null | undefined;
                title?: string | null | undefined;
                description?: string | null | undefined;
                media?: {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata: {
                        type: "File";
                    } | {
                        type: "Text";
                    } | {
                        type: "Image";
                        width: number;
                        height: number;
                    } | {
                        type: "Video";
                        width: number;
                        height: number;
                    } | {
                        type: "Audio";
                    };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                } | null | undefined;
                colour?: string | null | undefined;
            } | {
                type: "None";
            })[] | null | undefined;
            mentions?: string[] | null | undefined;
            replies?: string[] | null | undefined;
            reactions?: {
                [key: string]: string[];
            } | undefined;
            interactions?: {
                reactions?: string[] | null | undefined;
                restrict_reactions?: boolean | undefined;
            } | undefined;
            masquerade?: {
                name?: string | null | undefined;
                avatar?: string | null | undefined;
                colour?: string | null | undefined;
            } | null | undefined;
        }[];
        users: {
            _id: string;
            username: string;
            avatar?: {
                _id: string;
                tag: string;
                filename: string;
                metadata: {
                    type: "File";
                } | {
                    type: "Text";
                } | {
                    type: "Image";
                    width: number;
                    height: number;
                } | {
                    type: "Video";
                    width: number;
                    height: number;
                } | {
                    type: "Audio";
                };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
            } | null | undefined;
            relations?: {
                _id: string;
                status: "None" | "User" | "Friend" | "Outgoing" | "Incoming" | "Blocked" | "BlockedOther";
            }[] | null | undefined;
            badges?: number | null | undefined;
            status?: {
                text?: string | null | undefined;
                presence?: "Online" | "Idle" | "Focus" | "Busy" | "Invisible" | null | undefined;
            } | null | undefined;
            profile?: {
                content?: string | null | undefined;
                background?: {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata: {
                        type: "File";
                    } | {
                        type: "Text";
                    } | {
                        type: "Image";
                        width: number;
                        height: number;
                    } | {
                        type: "Video";
                        width: number;
                        height: number;
                    } | {
                        type: "Audio";
                    };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                } | null | undefined;
            } | null | undefined;
            flags?: number | null | undefined;
            privileged?: boolean | undefined;
            bot?: {
                owner: string;
            } | null | undefined;
            relationship?: "None" | "User" | "Friend" | "Outgoing" | "Incoming" | "Blocked" | "BlockedOther" | null | undefined;
            online?: boolean | null | undefined;
        }[];
        members?: {
            _id: {
                server: string;
                user: string;
            };
            joined_at: string;
            nickname?: string | null | undefined;
            avatar?: {
                _id: string;
                tag: string;
                filename: string;
                metadata: {
                    type: "File";
                } | {
                    type: "Text";
                } | {
                    type: "Image";
                    width: number;
                    height: number;
                } | {
                    type: "Video";
                    width: number;
                    height: number;
                } | {
                    type: "Audio";
                };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
            } | null | undefined;
            roles?: string[] | undefined;
            timeout?: string | null | undefined;
        }[] | null | undefined;
    }>;
}
//# sourceMappingURL=MessageCollection.d.ts.map