"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectStorage = void 0;
const store_1 = require("solid-js/store");
const hydration_1 = require("../hydration");
/**
 * Wrapper around Solid.js store
 */
class ObjectStorage {
    /**
     * Create new object storage
     */
    constructor() {
        const [store, setStore] = (0, store_1.createStore)({});
        this.store = store;
        this.set = setStore;
    }
    /**
     * Get object by ID
     * @param id ID
     * @returns Object
     */
    get(id) {
        return this.store[id];
    }
    /**
     * Hydrate some data into storage
     * @param id ID
     * @param type Hydration type
     * @param data Input Data
     */
    hydrate(id, type, data) {
        if (data) {
            this.set(id, (0, hydration_1.hydrate)(type, data));
        }
    }
}
exports.ObjectStorage = ObjectStorage;
