import { API } from "revolt-api";
import type { DataLogin, RevoltConfig } from "revolt-api";

import channelClassFactory from "./classes/Channel";
import emojiClassFactory from "./classes/Emoji";
import serverClassFactory from "./classes/Server";
import serverMemberClassFactory from "./classes/ServerMember";
import userClassFactory from "./classes/User";
import { EventClient } from "./events/eventClient";

type Obj<T extends (...args: any) => any> = InstanceType<ReturnType<T>>;
export type Emoji = Obj<typeof emojiClassFactory>;
export type Channel = Obj<typeof channelClassFactory>;
export type Server = Obj<typeof serverClassFactory>;
export type ServerMember = Obj<typeof serverMemberClassFactory>;
export type User = Obj<typeof userClassFactory>;
export type Session =
  | {
      token: string;
    }
  | string;
/**
 * Revolt.js Client
 */
export declare class Client {
  #private;
  readonly emojis: {
    new (
      id: string,
      data?:
        | {
            _id: string;
            parent:
              | {
                  type: "Server";
                  id: string;
                }
              | {
                  type: "Detached";
                };
            creator_id: string;
            name: string;
            animated?: boolean | undefined;
            nsfw?: boolean | undefined;
          }
        | undefined
    ): {
      readonly id: string;
      readonly createdAt: Date;
      readonly parent:
        | {
            type: "Server";
            id: string;
          }
        | {
            type: "Detached";
          }
      /**
       * Create Revolt.js Client
       */;
      readonly creator:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly name: string;
      readonly animated: boolean;
      readonly mature: boolean;
    };
    "__#6@#storage": import("./storage/ObjectStorage").ObjectStorage<
      import("./hydration/emoji").HydratedEmoji
    >;
    "__#6@#objects": Record<
      string,
      {
        readonly id: string;
        readonly createdAt: Date;
        readonly parent:
          | {
              type: "Server";
              id: string;
            }
          | {
              type: "Detached";
            }
        /**
         * Create Revolt.js Client
         */;
        readonly creator:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly name: string;
        readonly animated: boolean;
        readonly mature: boolean;
      }
    >;
    get(id: string):
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly parent:
            | {
                type: "Server";
                id: string;
              }
            | {
                type: "Detached";
              }
          /**
           * Create Revolt.js Client
           */;
          readonly creator:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly name: string;
          readonly animated: boolean;
          readonly mature: boolean;
        }
      | undefined;
    fetch(id: string): Promise<
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly parent:
            | {
                type: "Server";
                id: string;
              }
            | {
                type: "Detached";
              }
          /**
           * Create Revolt.js Client
           */;
          readonly creator:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly name: string;
          readonly animated: boolean;
          readonly mature: boolean;
        }
      | undefined
    >;
  };
  readonly channels: {
    new (
      id: string,
      data?:
        | {
            channel_type: "SavedMessages";
            _id: string;
            user: string;
          }
        | {
            channel_type: "DirectMessage";
            _id: string;
            active: boolean;
            recipients: string[];
            last_message_id?: string | null | undefined;
          }
        | {
            channel_type: "Group";
            _id: string;
            name: string;
            owner: string;
            description?: string | null | undefined;
            recipients: string[];
            icon?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            last_message_id?: string | null | undefined;
            permissions?: number | null | undefined;
            nsfw?: boolean | undefined;
          }
        | {
            channel_type: "TextChannel";
            _id: string;
            server: string;
            name: string;
            description?: string | null | undefined;
            icon?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            last_message_id?: string | null | undefined;
            default_permissions?:
              | {
                  a: number;
                  d: number;
                }
              | null
              | undefined;
            role_permissions?:
              | {
                  [key: string]: {
                    a: number;
                    d: number;
                  };
                }
              | undefined;
            nsfw?: boolean | undefined;
          }
        | {
            channel_type: "VoiceChannel";
            _id: string;
            server: string;
            name: string;
            description?: string | null | undefined;
            icon?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            default_permissions?:
              | {
                  a: number;
                  d: number;
                }
              | null
              | undefined;
            role_permissions?:
              | {
                  [key: string]: {
                    a: number;
                    d: number;
                  };
                }
              | undefined;
            nsfw?: boolean | undefined;
          }
        | undefined
    ): {
      readonly id: string;
      readonly createdAt: Date;
      readonly type:
        | "SavedMessages"
        | "DirectMessage"
        | "Group"
        | "TextChannel"
        | "VoiceChannel";
      readonly name: string;
      readonly description: string | undefined;
      readonly icon:
        | {
            _id: string;
            tag: string;
            filename: string;
            metadata:
              | {
                  type: "File";
                }
              | {
                  type: "Text";
                }
              | {
                  type: "Image";
                  width: number;
                  height: number;
                }
              | {
                  type: "Video";
                  width: number;
                  height: number;
                }
              | {
                  type: "Audio";
                };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
          }
        | undefined;
      readonly active: boolean;
      readonly recipients: {
        readonly id: string;
        readonly createdAt: Date;
        readonly username: string;
        readonly avatar:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly badges: import("./hydration/user").UserBadges;
        readonly status:
          | {
              text?: string | null | undefined;
              presence?:
                | "Online"
                | "Idle"
                | "Focus"
                | "Busy"
                | "Invisible"
                | null
                | undefined;
            }
          | undefined;
        readonly relationship:
          | "None"
          | "User"
          | "Friend"
          | "Outgoing"
          | "Incoming"
          | "Blocked"
          | "BlockedOther";
        readonly online: boolean;
        readonly privileged: boolean;
        readonly flags: import("./hydration/user").UserFlags;
        readonly bot:
          | {
              owner: string;
            }
          | undefined;
      }[];
      readonly user:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly owner:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly server:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly permissions: number | undefined;
      readonly defaultPermissions:
        | {
            a: number;
            d: number;
          }
        | undefined;
      readonly rolePermissions:
        | Record<
            string,
            {
              a: number;
              d: number;
            }
          >
        | undefined;
      readonly mature: boolean;
      readonly lastMessageId: string | undefined;
    };
    "__#5@#storage": import("./storage/ObjectStorage").ObjectStorage<
      import("./hydration/channel").HydratedChannel
    >;
    "__#5@#objects": Record<
      string,
      {
        readonly id: string;
        readonly createdAt: Date;
        readonly type:
          | "SavedMessages"
          | "DirectMessage"
          | "Group"
          | "TextChannel"
          | "VoiceChannel";
        readonly name: string;
        readonly description: string | undefined;
        readonly icon:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly active: boolean;
        readonly recipients: {
          readonly id: string;
          readonly createdAt: Date;
          readonly username: string;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly badges: import("./hydration/user").UserBadges;
          readonly status:
            | {
                text?: string | null | undefined;
                presence?:
                  | "Online"
                  | "Idle"
                  | "Focus"
                  | "Busy"
                  | "Invisible"
                  | null
                  | undefined;
              }
            | undefined;
          readonly relationship:
            | "None"
            | "User"
            | "Friend"
            | "Outgoing"
            | "Incoming"
            | "Blocked"
            | "BlockedOther";
          readonly online: boolean;
          readonly privileged: boolean;
          readonly flags: import("./hydration/user").UserFlags;
          readonly bot:
            | {
                owner: string;
              }
            | undefined;
        }[];
        readonly user:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly owner:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly server:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly permissions: number | undefined;
        readonly defaultPermissions:
          | {
              a: number;
              d: number;
            }
          | undefined;
        readonly rolePermissions:
          | Record<
              string,
              {
                a: number;
                d: number;
              }
            >
          | undefined;
        readonly mature: boolean;
        readonly lastMessageId: string | undefined;
      }
    >;
    get(id: string):
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly type:
            | "SavedMessages"
            | "DirectMessage"
            | "Group"
            | "TextChannel"
            | "VoiceChannel";
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly active: boolean;
          readonly recipients: {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }[];
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly permissions: number | undefined;
          readonly defaultPermissions:
            | {
                a: number;
                d: number;
              }
            | undefined;
          readonly rolePermissions:
            | Record<
                string,
                {
                  a: number;
                  d: number;
                }
              >
            | undefined;
          readonly mature: boolean;
          readonly lastMessageId: string | undefined;
        }
      | undefined;
    fetch(id: string): Promise<
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly type:
            | "SavedMessages"
            | "DirectMessage"
            | "Group"
            | "TextChannel"
            | "VoiceChannel";
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly active: boolean;
          readonly recipients: {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }[];
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly permissions: number | undefined;
          readonly defaultPermissions:
            | {
                a: number;
                d: number;
              }
            | undefined;
          readonly rolePermissions:
            | Record<
                string,
                {
                  a: number;
                  d: number;
                }
              >
            | undefined;
          readonly mature: boolean;
          readonly lastMessageId: string | undefined;
        }
      | undefined
    >;
  };
  readonly users: {
    new (
      id: string,
      data?:
        | {
            _id: string;
            username: string;
            avatar?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            relations?:
              | {
                  _id: string;
                  status:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                }[]
              | null
              | undefined;
            badges?: number | null | undefined;
            status?:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | null
              | undefined;
            profile?:
              | {
                  content?: string | null | undefined;
                  background?:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | null
                    | undefined;
                }
              | null
              | undefined;
            flags?: number | null | undefined;
            privileged?: boolean | undefined;
            bot?:
              | {
                  owner: string;
                }
              | null
              | undefined;
            relationship?:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther"
              | null
              | undefined;
            online?: boolean | null | undefined;
          }
        | undefined
    ): {
      readonly id: string;
      readonly createdAt: Date;
      readonly username: string;
      readonly avatar:
        | {
            _id: string;
            tag: string;
            filename: string;
            metadata:
              | {
                  type: "File";
                }
              | {
                  type: "Text";
                }
              | {
                  type: "Image";
                  width: number;
                  height: number;
                }
              | {
                  type: "Video";
                  width: number;
                  height: number;
                }
              | {
                  type: "Audio";
                };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
          }
        | undefined;
      readonly badges: import("./hydration/user").UserBadges;
      readonly status:
        | {
            text?: string | null | undefined;
            presence?:
              | "Online"
              | "Idle"
              | "Focus"
              | "Busy"
              | "Invisible"
              | null
              | undefined;
          }
        | undefined;
      readonly relationship:
        | "None"
        | "User"
        | "Friend"
        | "Outgoing"
        | "Incoming"
        | "Blocked"
        | "BlockedOther";
      readonly online: boolean;
      readonly privileged: boolean;
      readonly flags: import("./hydration/user").UserFlags;
      readonly bot:
        | {
            owner: string;
          }
        | undefined;
    };
    "__#9@#storage": import("./storage/ObjectStorage").ObjectStorage<
      import("./hydration/user").HydratedUser
    >;
    "__#9@#objects": Record<
      string,
      {
        readonly id: string;
        readonly createdAt: Date;
        readonly username: string;
        readonly avatar:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly badges: import("./hydration/user").UserBadges;
        readonly status:
          | {
              text?: string | null | undefined;
              presence?:
                | "Online"
                | "Idle"
                | "Focus"
                | "Busy"
                | "Invisible"
                | null
                | undefined;
            }
          | undefined;
        readonly relationship:
          | "None"
          | "User"
          | "Friend"
          | "Outgoing"
          | "Incoming"
          | "Blocked"
          | "BlockedOther";
        readonly online: boolean;
        readonly privileged: boolean;
        readonly flags: import("./hydration/user").UserFlags;
        readonly bot:
          | {
              owner: string;
            }
          | undefined;
      }
    >;
    get(id: string):
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly username: string;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly badges: import("./hydration/user").UserBadges;
          readonly status:
            | {
                text?: string | null | undefined;
                presence?:
                  | "Online"
                  | "Idle"
                  | "Focus"
                  | "Busy"
                  | "Invisible"
                  | null
                  | undefined;
              }
            | undefined;
          readonly relationship:
            | "None"
            | "User"
            | "Friend"
            | "Outgoing"
            | "Incoming"
            | "Blocked"
            | "BlockedOther";
          readonly online: boolean;
          readonly privileged: boolean;
          readonly flags: import("./hydration/user").UserFlags;
          readonly bot:
            | {
                owner: string;
              }
            | undefined;
        }
      | undefined;
    fetch(id: string): Promise<
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly username: string;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly badges: import("./hydration/user").UserBadges;
          readonly status:
            | {
                text?: string | null | undefined;
                presence?:
                  | "Online"
                  | "Idle"
                  | "Focus"
                  | "Busy"
                  | "Invisible"
                  | null
                  | undefined;
              }
            | undefined;
          readonly relationship:
            | "None"
            | "User"
            | "Friend"
            | "Outgoing"
            | "Incoming"
            | "Blocked"
            | "BlockedOther";
          readonly online: boolean;
          readonly privileged: boolean;
          readonly flags: import("./hydration/user").UserFlags;
          readonly bot:
            | {
                owner: string;
              }
            | undefined;
        }
      | undefined
    >;
  };
  readonly servers: {
    new (
      id: string,
      data?:
        | {
            _id: string;
            owner: string;
            name: string;
            description?: string | null | undefined;
            channels: string[];
            categories?:
              | {
                  id: string;
                  title: string;
                  channels: string[];
                }[]
              | null
              | undefined;
            system_messages?:
              | {
                  user_joined?: string | null | undefined;
                  user_left?: string | null | undefined;
                  user_kicked?: string | null | undefined;
                  user_banned?: string | null | undefined;
                }
              | null
              | undefined;
            roles?:
              | {
                  [key: string]: {
                    name: string;
                    permissions: {
                      a: number;
                      d: number;
                    };
                    colour?: string | null | undefined;
                    hoist?: boolean | undefined;
                    rank?: number | undefined;
                  };
                }
              | undefined;
            default_permissions: number;
            icon?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            banner?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            flags?: number | null | undefined;
            nsfw?: boolean | undefined;
            analytics?: boolean | undefined;
            discoverable?: boolean | undefined;
          }
        | undefined
    ): {
      readonly id: string;
      readonly createdAt: Date;
      readonly owner:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly name: string;
      readonly description: string | undefined;
      readonly icon:
        | {
            _id: string;
            tag: string;
            filename: string;
            metadata:
              | {
                  type: "File";
                }
              | {
                  type: "Text";
                }
              | {
                  type: "Image";
                  width: number;
                  height: number;
                }
              | {
                  type: "Video";
                  width: number;
                  height: number;
                }
              | {
                  type: "Audio";
                };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
          }
        | undefined;
      readonly banner:
        | {
            _id: string;
            tag: string;
            filename: string;
            metadata:
              | {
                  type: "File";
                }
              | {
                  type: "Text";
                }
              | {
                  type: "Image";
                  width: number;
                  height: number;
                }
              | {
                  type: "Video";
                  width: number;
                  height: number;
                }
              | {
                  type: "Audio";
                };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
          }
        | undefined;
      readonly channels: {
        readonly id: string;
        readonly createdAt: Date;
        readonly type:
          | "SavedMessages"
          | "DirectMessage"
          | "Group"
          | "TextChannel"
          | "VoiceChannel";
        readonly name: string;
        readonly description: string | undefined;
        readonly icon:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly active: boolean;
        readonly recipients: {
          readonly id: string;
          readonly createdAt: Date;
          readonly username: string;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly badges: import("./hydration/user").UserBadges;
          readonly status:
            | {
                text?: string | null | undefined;
                presence?:
                  | "Online"
                  | "Idle"
                  | "Focus"
                  | "Busy"
                  | "Invisible"
                  | null
                  | undefined;
              }
            | undefined;
          readonly relationship:
            | "None"
            | "User"
            | "Friend"
            | "Outgoing"
            | "Incoming"
            | "Blocked"
            | "BlockedOther";
          readonly online: boolean;
          readonly privileged: boolean;
          readonly flags: import("./hydration/user").UserFlags;
          readonly bot:
            | {
                owner: string;
              }
            | undefined;
        }[];
        readonly user:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly owner:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly server:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly permissions: number | undefined;
        readonly defaultPermissions:
          | {
              a: number;
              d: number;
            }
          | undefined;
        readonly rolePermissions:
          | Record<
              string,
              {
                a: number;
                d: number;
              }
            >
          | undefined;
        readonly mature: boolean;
        readonly lastMessageId: string | undefined;
      }[];
      readonly categories:
        | {
            id: string;
            title: string;
            channels: string[];
          }[]
        | undefined;
      readonly systemMessages:
        | {
            user_joined?: string | null | undefined;
            user_left?: string | null | undefined;
            user_kicked?: string | null | undefined;
            user_banned?: string | null | undefined;
          }
        | undefined;
      readonly roles:
        | import("@solid-primitives/map").ReactiveMap<
            string,
            {
              name: string;
              permissions: {
                a: number;
                d: number;
              };
              colour?: string | null | undefined;
              hoist?: boolean | undefined;
              rank?: number | undefined;
            }
          >
        | undefined;
      readonly defaultPermissions: number;
      readonly flags: import("./hydration/server").ServerFlags;
      readonly analytics: boolean;
      readonly discoverable: boolean;
      readonly mature: boolean;
      readonly orderedChannels: (Omit<
        {
          id: string;
          title: string;
          channels: string[];
        },
        "channels"
      > & {
        channels: {
          readonly id: string;
          readonly createdAt: Date;
          readonly type:
            | "SavedMessages"
            | "DirectMessage"
            | "Group"
            | "TextChannel"
            | "VoiceChannel";
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly active: boolean;
          readonly recipients: {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }[];
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly permissions: number | undefined;
          readonly defaultPermissions:
            | {
                a: number;
                d: number;
              }
            | undefined;
          readonly rolePermissions:
            | Record<
                string,
                {
                  a: number;
                  d: number;
                }
              >
            | undefined;
          readonly mature: boolean;
          readonly lastMessageId: string | undefined;
        }[];
      })[];
      readonly defaultChannel:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly type:
              | "SavedMessages"
              | "DirectMessage"
              | "Group"
              | "TextChannel"
              | "VoiceChannel";
            readonly name: string;
            readonly description: string | undefined;
            readonly icon:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly active: boolean;
            readonly recipients: {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }[];
            readonly user:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly owner:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly server:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly permissions: number | undefined;
            readonly defaultPermissions:
              | {
                  a: number;
                  d: number;
                }
              | undefined;
            readonly rolePermissions:
              | Record<
                  string,
                  {
                    a: number;
                    d: number;
                  }
                >
              | undefined;
            readonly mature: boolean;
            readonly lastMessageId: string | undefined;
          }
        | undefined;
      readonly orderedRoles: {
        name: string;
        permissions: {
          a: number;
          d: number;
        };
        colour?: string | null | undefined;
        hoist?: boolean | undefined;
        rank?: number | undefined;
        id: string;
      }[];
    };
    "__#7@#storage": import("./storage/ObjectStorage").ObjectStorage<
      import("./hydration/server").HydratedServer
    >;
    "__#7@#objects": Record<
      string,
      {
        readonly id: string;
        readonly createdAt: Date;
        readonly owner:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly name: string;
        readonly description: string | undefined;
        readonly icon:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly banner:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly channels: {
          readonly id: string;
          readonly createdAt: Date;
          readonly type:
            | "SavedMessages"
            | "DirectMessage"
            | "Group"
            | "TextChannel"
            | "VoiceChannel";
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly active: boolean;
          readonly recipients: {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }[];
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly permissions: number | undefined;
          readonly defaultPermissions:
            | {
                a: number;
                d: number;
              }
            | undefined;
          readonly rolePermissions:
            | Record<
                string,
                {
                  a: number;
                  d: number;
                }
              >
            | undefined;
          readonly mature: boolean;
          readonly lastMessageId: string | undefined;
        }[];
        readonly categories:
          | {
              id: string;
              title: string;
              channels: string[];
            }[]
          | undefined;
        readonly systemMessages:
          | {
              user_joined?: string | null | undefined;
              user_left?: string | null | undefined;
              user_kicked?: string | null | undefined;
              user_banned?: string | null | undefined;
            }
          | undefined;
        readonly roles:
          | import("@solid-primitives/map").ReactiveMap<
              string,
              {
                name: string;
                permissions: {
                  a: number;
                  d: number;
                };
                colour?: string | null | undefined;
                hoist?: boolean | undefined;
                rank?: number | undefined;
              }
            >
          | undefined;
        readonly defaultPermissions: number;
        readonly flags: import("./hydration/server").ServerFlags;
        readonly analytics: boolean;
        readonly discoverable: boolean;
        readonly mature: boolean;
        readonly orderedChannels: (Omit<
          {
            id: string;
            title: string;
            channels: string[];
          },
          "channels"
        > & {
          channels: {
            readonly id: string;
            readonly createdAt: Date;
            readonly type:
              | "SavedMessages"
              | "DirectMessage"
              | "Group"
              | "TextChannel"
              | "VoiceChannel";
            readonly name: string;
            readonly description: string | undefined;
            readonly icon:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly active: boolean;
            readonly recipients: {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }[];
            readonly user:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly owner:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly server:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly permissions: number | undefined;
            readonly defaultPermissions:
              | {
                  a: number;
                  d: number;
                }
              | undefined;
            readonly rolePermissions:
              | Record<
                  string,
                  {
                    a: number;
                    d: number;
                  }
                >
              | undefined;
            readonly mature: boolean;
            readonly lastMessageId: string | undefined;
          }[];
        })[];
        readonly defaultChannel:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly type:
                | "SavedMessages"
                | "DirectMessage"
                | "Group"
                | "TextChannel"
                | "VoiceChannel";
              readonly name: string;
              readonly description: string | undefined;
              readonly icon:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly active: boolean;
              readonly recipients: {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }[];
              readonly user:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly owner:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly server:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly permissions: number | undefined;
              readonly defaultPermissions:
                | {
                    a: number;
                    d: number;
                  }
                | undefined;
              readonly rolePermissions:
                | Record<
                    string,
                    {
                      a: number;
                      d: number;
                    }
                  >
                | undefined;
              readonly mature: boolean;
              readonly lastMessageId: string | undefined;
            }
          | undefined;
        readonly orderedRoles: {
          name: string;
          permissions: {
            a: number;
            d: number;
          };
          colour?: string | null | undefined;
          hoist?: boolean | undefined;
          rank?: number | undefined;
          id: string;
        }[];
      }
    >;
    get(id: string):
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly banner:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly channels: {
            readonly id: string;
            readonly createdAt: Date;
            readonly type:
              | "SavedMessages"
              | "DirectMessage"
              | "Group"
              | "TextChannel"
              | "VoiceChannel";
            readonly name: string;
            readonly description: string | undefined;
            readonly icon:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly active: boolean;
            readonly recipients: {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }[];
            readonly user:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly owner:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly server:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly permissions: number | undefined;
            readonly defaultPermissions:
              | {
                  a: number;
                  d: number;
                }
              | undefined;
            readonly rolePermissions:
              | Record<
                  string,
                  {
                    a: number;
                    d: number;
                  }
                >
              | undefined;
            readonly mature: boolean;
            readonly lastMessageId: string | undefined;
          }[];
          readonly categories:
            | {
                id: string;
                title: string;
                channels: string[];
              }[]
            | undefined;
          readonly systemMessages:
            | {
                user_joined?: string | null | undefined;
                user_left?: string | null | undefined;
                user_kicked?: string | null | undefined;
                user_banned?: string | null | undefined;
              }
            | undefined;
          readonly roles:
            | import("@solid-primitives/map").ReactiveMap<
                string,
                {
                  name: string;
                  permissions: {
                    a: number;
                    d: number;
                  };
                  colour?: string | null | undefined;
                  hoist?: boolean | undefined;
                  rank?: number | undefined;
                }
              >
            | undefined;
          readonly defaultPermissions: number;
          readonly flags: import("./hydration/server").ServerFlags;
          readonly analytics: boolean;
          readonly discoverable: boolean;
          readonly mature: boolean;
          readonly orderedChannels: (Omit<
            {
              id: string;
              title: string;
              channels: string[];
            },
            "channels"
          > & {
            channels: {
              readonly id: string;
              readonly createdAt: Date;
              readonly type:
                | "SavedMessages"
                | "DirectMessage"
                | "Group"
                | "TextChannel"
                | "VoiceChannel";
              readonly name: string;
              readonly description: string | undefined;
              readonly icon:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly active: boolean;
              readonly recipients: {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }[];
              readonly user:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly owner:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly server:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly permissions: number | undefined;
              readonly defaultPermissions:
                | {
                    a: number;
                    d: number;
                  }
                | undefined;
              readonly rolePermissions:
                | Record<
                    string,
                    {
                      a: number;
                      d: number;
                    }
                  >
                | undefined;
              readonly mature: boolean;
              readonly lastMessageId: string | undefined;
            }[];
          })[];
          readonly defaultChannel:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly type:
                  | "SavedMessages"
                  | "DirectMessage"
                  | "Group"
                  | "TextChannel"
                  | "VoiceChannel";
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly active: boolean;
                readonly recipients: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }[];
                readonly user:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly server:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly permissions: number | undefined;
                readonly defaultPermissions:
                  | {
                      a: number;
                      d: number;
                    }
                  | undefined;
                readonly rolePermissions:
                  | Record<
                      string,
                      {
                        a: number;
                        d: number;
                      }
                    >
                  | undefined;
                readonly mature: boolean;
                readonly lastMessageId: string | undefined;
              }
            | undefined;
          readonly orderedRoles: {
            name: string;
            permissions: {
              a: number;
              d: number;
            };
            colour?: string | null | undefined;
            hoist?: boolean | undefined;
            rank?: number | undefined;
            id: string;
          }[];
        }
      | undefined;
    fetch(id: string): Promise<
      | {
          readonly id: string;
          readonly createdAt: Date;
          readonly owner:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly name: string;
          readonly description: string | undefined;
          readonly icon:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly banner:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly channels: {
            readonly id: string;
            readonly createdAt: Date;
            readonly type:
              | "SavedMessages"
              | "DirectMessage"
              | "Group"
              | "TextChannel"
              | "VoiceChannel";
            readonly name: string;
            readonly description: string | undefined;
            readonly icon:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly active: boolean;
            readonly recipients: {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }[];
            readonly user:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly owner:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly server:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly permissions: number | undefined;
            readonly defaultPermissions:
              | {
                  a: number;
                  d: number;
                }
              | undefined;
            readonly rolePermissions:
              | Record<
                  string,
                  {
                    a: number;
                    d: number;
                  }
                >
              | undefined;
            readonly mature: boolean;
            readonly lastMessageId: string | undefined;
          }[];
          readonly categories:
            | {
                id: string;
                title: string;
                channels: string[];
              }[]
            | undefined;
          readonly systemMessages:
            | {
                user_joined?: string | null | undefined;
                user_left?: string | null | undefined;
                user_kicked?: string | null | undefined;
                user_banned?: string | null | undefined;
              }
            | undefined;
          readonly roles:
            | import("@solid-primitives/map").ReactiveMap<
                string,
                {
                  name: string;
                  permissions: {
                    a: number;
                    d: number;
                  };
                  colour?: string | null | undefined;
                  hoist?: boolean | undefined;
                  rank?: number | undefined;
                }
              >
            | undefined;
          readonly defaultPermissions: number;
          readonly flags: import("./hydration/server").ServerFlags;
          readonly analytics: boolean;
          readonly discoverable: boolean;
          readonly mature: boolean;
          readonly orderedChannels: (Omit<
            {
              id: string;
              title: string;
              channels: string[];
            },
            "channels"
          > & {
            channels: {
              readonly id: string;
              readonly createdAt: Date;
              readonly type:
                | "SavedMessages"
                | "DirectMessage"
                | "Group"
                | "TextChannel"
                | "VoiceChannel";
              readonly name: string;
              readonly description: string | undefined;
              readonly icon:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly active: boolean;
              readonly recipients: {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }[];
              readonly user:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly owner:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly server:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly permissions: number | undefined;
              readonly defaultPermissions:
                | {
                    a: number;
                    d: number;
                  }
                | undefined;
              readonly rolePermissions:
                | Record<
                    string,
                    {
                      a: number;
                      d: number;
                    }
                  >
                | undefined;
              readonly mature: boolean;
              readonly lastMessageId: string | undefined;
            }[];
          })[];
          readonly defaultChannel:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly type:
                  | "SavedMessages"
                  | "DirectMessage"
                  | "Group"
                  | "TextChannel"
                  | "VoiceChannel";
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly active: boolean;
                readonly recipients: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }[];
                readonly user:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly server:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly permissions: number | undefined;
                readonly defaultPermissions:
                  | {
                      a: number;
                      d: number;
                    }
                  | undefined;
                readonly rolePermissions:
                  | Record<
                      string,
                      {
                        a: number;
                        d: number;
                      }
                    >
                  | undefined;
                readonly mature: boolean;
                readonly lastMessageId: string | undefined;
              }
            | undefined;
          readonly orderedRoles: {
            name: string;
            permissions: {
              a: number;
              d: number;
            };
            colour?: string | null | undefined;
            hoist?: boolean | undefined;
            rank?: number | undefined;
            id: string;
          }[];
        }
      | undefined
    >;
  };
  readonly serverMembers: {
    new (
      id: {
        server: string;
        user: string;
      },
      data?:
        | {
            _id: {
              server: string;
              user: string;
            };
            joined_at: string;
            nickname?: string | null | undefined;
            avatar?:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | null
              | undefined;
            roles?: string[] | undefined;
            timeout?: string | null | undefined;
          }
        | undefined
    ): {
      readonly id: {
        server: string;
        user: string;
      };
      readonly server:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly owner:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }
              | undefined;
            readonly name: string;
            readonly description: string | undefined;
            readonly icon:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly banner:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly channels: {
              readonly id: string;
              readonly createdAt: Date;
              readonly type:
                | "SavedMessages"
                | "DirectMessage"
                | "Group"
                | "TextChannel"
                | "VoiceChannel";
              readonly name: string;
              readonly description: string | undefined;
              readonly icon:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly active: boolean;
              readonly recipients: {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }[];
              readonly user:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly owner:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly server:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly permissions: number | undefined;
              readonly defaultPermissions:
                | {
                    a: number;
                    d: number;
                  }
                | undefined;
              readonly rolePermissions:
                | Record<
                    string,
                    {
                      a: number;
                      d: number;
                    }
                  >
                | undefined;
              readonly mature: boolean;
              readonly lastMessageId: string | undefined;
            }[];
            readonly categories:
              | {
                  id: string;
                  title: string;
                  channels: string[];
                }[]
              | undefined;
            readonly systemMessages:
              | {
                  user_joined?: string | null | undefined;
                  user_left?: string | null | undefined;
                  user_kicked?: string | null | undefined;
                  user_banned?: string | null | undefined;
                }
              | undefined;
            readonly roles:
              | import("@solid-primitives/map").ReactiveMap<
                  string,
                  {
                    name: string;
                    permissions: {
                      a: number;
                      d: number;
                    };
                    colour?: string | null | undefined;
                    hoist?: boolean | undefined;
                    rank?: number | undefined;
                  }
                >
              | undefined;
            readonly defaultPermissions: number;
            readonly flags: import("./hydration/server").ServerFlags;
            readonly analytics: boolean;
            readonly discoverable: boolean;
            readonly mature: boolean;
            readonly orderedChannels: (Omit<
              {
                id: string;
                title: string;
                channels: string[];
              },
              "channels"
            > & {
              channels: {
                readonly id: string;
                readonly createdAt: Date;
                readonly type:
                  | "SavedMessages"
                  | "DirectMessage"
                  | "Group"
                  | "TextChannel"
                  | "VoiceChannel";
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly active: boolean;
                readonly recipients: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }[];
                readonly user:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly server:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly permissions: number | undefined;
                readonly defaultPermissions:
                  | {
                      a: number;
                      d: number;
                    }
                  | undefined;
                readonly rolePermissions:
                  | Record<
                      string,
                      {
                        a: number;
                        d: number;
                      }
                    >
                  | undefined;
                readonly mature: boolean;
                readonly lastMessageId: string | undefined;
              }[];
            })[];
            readonly defaultChannel:
              | {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly type:
                    | "SavedMessages"
                    | "DirectMessage"
                    | "Group"
                    | "TextChannel"
                    | "VoiceChannel";
                  readonly name: string;
                  readonly description: string | undefined;
                  readonly icon:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly active: boolean;
                  readonly recipients: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }[];
                  readonly user:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly owner:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly server:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly permissions: number | undefined;
                  readonly defaultPermissions:
                    | {
                        a: number;
                        d: number;
                      }
                    | undefined;
                  readonly rolePermissions:
                    | Record<
                        string,
                        {
                          a: number;
                          d: number;
                        }
                      >
                    | undefined;
                  readonly mature: boolean;
                  readonly lastMessageId: string | undefined;
                }
              | undefined;
            readonly orderedRoles: {
              name: string;
              permissions: {
                a: number;
                d: number;
              };
              colour?: string | null | undefined;
              hoist?: boolean | undefined;
              rank?: number | undefined;
              id: string;
            }[];
          }
        | undefined;
      readonly user:
        | {
            readonly id: string;
            readonly createdAt: Date;
            readonly username: string;
            readonly avatar:
              | {
                  _id: string;
                  tag: string;
                  filename: string;
                  metadata:
                    | {
                        type: "File";
                      }
                    | {
                        type: "Text";
                      }
                    | {
                        type: "Image";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Video";
                        width: number;
                        height: number;
                      }
                    | {
                        type: "Audio";
                      };
                  content_type: string;
                  size: number;
                  deleted?: boolean | null | undefined;
                  reported?: boolean | null | undefined;
                  message_id?: string | null | undefined;
                  user_id?: string | null | undefined;
                  server_id?: string | null | undefined;
                  object_id?: string | null | undefined;
                }
              | undefined;
            readonly badges: import("./hydration/user").UserBadges;
            readonly status:
              | {
                  text?: string | null | undefined;
                  presence?:
                    | "Online"
                    | "Idle"
                    | "Focus"
                    | "Busy"
                    | "Invisible"
                    | null
                    | undefined;
                }
              | undefined;
            readonly relationship:
              | "None"
              | "User"
              | "Friend"
              | "Outgoing"
              | "Incoming"
              | "Blocked"
              | "BlockedOther";
            readonly online: boolean;
            readonly privileged: boolean;
            readonly flags: import("./hydration/user").UserFlags;
            readonly bot:
              | {
                  owner: string;
                }
              | undefined;
          }
        | undefined;
      readonly joinedAt: Date;
      readonly nickname: string | undefined;
      readonly avatar:
        | {
            _id: string;
            tag: string;
            filename: string;
            metadata:
              | {
                  type: "File";
                }
              | {
                  type: "Text";
                }
              | {
                  type: "Image";
                  width: number;
                  height: number;
                }
              | {
                  type: "Video";
                  width: number;
                  height: number;
                }
              | {
                  type: "Audio";
                };
            content_type: string;
            size: number;
            deleted?: boolean | null | undefined;
            reported?: boolean | null | undefined;
            message_id?: string | null | undefined;
            user_id?: string | null | undefined;
            server_id?: string | null | undefined;
            object_id?: string | null | undefined;
          }
        | undefined;
      readonly roles: string[] | undefined;
      readonly timeout: Date | undefined;
    };
    "__#8@#storage": import("./storage/ObjectStorage").ObjectStorage<
      import("./hydration/serverMember").HydratedServerMember
    >;
    "__#8@#objects": Record<
      string,
      {
        readonly id: {
          server: string;
          user: string;
        };
        readonly server:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly owner:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }
                | undefined;
              readonly name: string;
              readonly description: string | undefined;
              readonly icon:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly banner:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly channels: {
                readonly id: string;
                readonly createdAt: Date;
                readonly type:
                  | "SavedMessages"
                  | "DirectMessage"
                  | "Group"
                  | "TextChannel"
                  | "VoiceChannel";
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly active: boolean;
                readonly recipients: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly username: string;
                  readonly avatar:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly badges: import("./hydration/user").UserBadges;
                  readonly status:
                    | {
                        text?: string | null | undefined;
                        presence?:
                          | "Online"
                          | "Idle"
                          | "Focus"
                          | "Busy"
                          | "Invisible"
                          | null
                          | undefined;
                      }
                    | undefined;
                  readonly relationship:
                    | "None"
                    | "User"
                    | "Friend"
                    | "Outgoing"
                    | "Incoming"
                    | "Blocked"
                    | "BlockedOther";
                  readonly online: boolean;
                  readonly privileged: boolean;
                  readonly flags: import("./hydration/user").UserFlags;
                  readonly bot:
                    | {
                        owner: string;
                      }
                    | undefined;
                }[];
                readonly user:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly server:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly permissions: number | undefined;
                readonly defaultPermissions:
                  | {
                      a: number;
                      d: number;
                    }
                  | undefined;
                readonly rolePermissions:
                  | Record<
                      string,
                      {
                        a: number;
                        d: number;
                      }
                    >
                  | undefined;
                readonly mature: boolean;
                readonly lastMessageId: string | undefined;
              }[];
              readonly categories:
                | {
                    id: string;
                    title: string;
                    channels: string[];
                  }[]
                | undefined;
              readonly systemMessages:
                | {
                    user_joined?: string | null | undefined;
                    user_left?: string | null | undefined;
                    user_kicked?: string | null | undefined;
                    user_banned?: string | null | undefined;
                  }
                | undefined;
              readonly roles:
                | import("@solid-primitives/map").ReactiveMap<
                    string,
                    {
                      name: string;
                      permissions: {
                        a: number;
                        d: number;
                      };
                      colour?: string | null | undefined;
                      hoist?: boolean | undefined;
                      rank?: number | undefined;
                    }
                  >
                | undefined;
              readonly defaultPermissions: number;
              readonly flags: import("./hydration/server").ServerFlags;
              readonly analytics: boolean;
              readonly discoverable: boolean;
              readonly mature: boolean;
              readonly orderedChannels: (Omit<
                {
                  id: string;
                  title: string;
                  channels: string[];
                },
                "channels"
              > & {
                channels: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly type:
                    | "SavedMessages"
                    | "DirectMessage"
                    | "Group"
                    | "TextChannel"
                    | "VoiceChannel";
                  readonly name: string;
                  readonly description: string | undefined;
                  readonly icon:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly active: boolean;
                  readonly recipients: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }[];
                  readonly user:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly owner:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly server:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly permissions: number | undefined;
                  readonly defaultPermissions:
                    | {
                        a: number;
                        d: number;
                      }
                    | undefined;
                  readonly rolePermissions:
                    | Record<
                        string,
                        {
                          a: number;
                          d: number;
                        }
                      >
                    | undefined;
                  readonly mature: boolean;
                  readonly lastMessageId: string | undefined;
                }[];
              })[];
              readonly defaultChannel:
                | {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly type:
                      | "SavedMessages"
                      | "DirectMessage"
                      | "Group"
                      | "TextChannel"
                      | "VoiceChannel";
                    readonly name: string;
                    readonly description: string | undefined;
                    readonly icon:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly active: boolean;
                    readonly recipients: {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }[];
                    readonly user:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly owner:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly server:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly permissions: number | undefined;
                    readonly defaultPermissions:
                      | {
                          a: number;
                          d: number;
                        }
                      | undefined;
                    readonly rolePermissions:
                      | Record<
                          string,
                          {
                            a: number;
                            d: number;
                          }
                        >
                      | undefined;
                    readonly mature: boolean;
                    readonly lastMessageId: string | undefined;
                  }
                | undefined;
              readonly orderedRoles: {
                name: string;
                permissions: {
                  a: number;
                  d: number;
                };
                colour?: string | null | undefined;
                hoist?: boolean | undefined;
                rank?: number | undefined;
                id: string;
              }[];
            }
          | undefined;
        readonly user:
          | {
              readonly id: string;
              readonly createdAt: Date;
              readonly username: string;
              readonly avatar:
                | {
                    _id: string;
                    tag: string;
                    filename: string;
                    metadata:
                      | {
                          type: "File";
                        }
                      | {
                          type: "Text";
                        }
                      | {
                          type: "Image";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Video";
                          width: number;
                          height: number;
                        }
                      | {
                          type: "Audio";
                        };
                    content_type: string;
                    size: number;
                    deleted?: boolean | null | undefined;
                    reported?: boolean | null | undefined;
                    message_id?: string | null | undefined;
                    user_id?: string | null | undefined;
                    server_id?: string | null | undefined;
                    object_id?: string | null | undefined;
                  }
                | undefined;
              readonly badges: import("./hydration/user").UserBadges;
              readonly status:
                | {
                    text?: string | null | undefined;
                    presence?:
                      | "Online"
                      | "Idle"
                      | "Focus"
                      | "Busy"
                      | "Invisible"
                      | null
                      | undefined;
                  }
                | undefined;
              readonly relationship:
                | "None"
                | "User"
                | "Friend"
                | "Outgoing"
                | "Incoming"
                | "Blocked"
                | "BlockedOther";
              readonly online: boolean;
              readonly privileged: boolean;
              readonly flags: import("./hydration/user").UserFlags;
              readonly bot:
                | {
                    owner: string;
                  }
                | undefined;
            }
          | undefined;
        readonly joinedAt: Date;
        readonly nickname: string | undefined;
        readonly avatar:
          | {
              _id: string;
              tag: string;
              filename: string;
              metadata:
                | {
                    type: "File";
                  }
                | {
                    type: "Text";
                  }
                | {
                    type: "Image";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Video";
                    width: number;
                    height: number;
                  }
                | {
                    type: "Audio";
                  };
              content_type: string;
              size: number;
              deleted?: boolean | null | undefined;
              reported?: boolean | null | undefined;
              message_id?: string | null | undefined;
              user_id?: string | null | undefined;
              server_id?: string | null | undefined;
              object_id?: string | null | undefined;
            }
          | undefined;
        readonly roles: string[] | undefined;
        readonly timeout: Date | undefined;
      }
    >;
    keyToString: (key: { server: string; user: string }) => string;
    get(id: { server: string; user: string }):
      | {
          readonly id: {
            server: string;
            user: string;
          };
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly banner:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly channels: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly type:
                    | "SavedMessages"
                    | "DirectMessage"
                    | "Group"
                    | "TextChannel"
                    | "VoiceChannel";
                  readonly name: string;
                  readonly description: string | undefined;
                  readonly icon:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly active: boolean;
                  readonly recipients: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }[];
                  readonly user:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly owner:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly server:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly permissions: number | undefined;
                  readonly defaultPermissions:
                    | {
                        a: number;
                        d: number;
                      }
                    | undefined;
                  readonly rolePermissions:
                    | Record<
                        string,
                        {
                          a: number;
                          d: number;
                        }
                      >
                    | undefined;
                  readonly mature: boolean;
                  readonly lastMessageId: string | undefined;
                }[];
                readonly categories:
                  | {
                      id: string;
                      title: string;
                      channels: string[];
                    }[]
                  | undefined;
                readonly systemMessages:
                  | {
                      user_joined?: string | null | undefined;
                      user_left?: string | null | undefined;
                      user_kicked?: string | null | undefined;
                      user_banned?: string | null | undefined;
                    }
                  | undefined;
                readonly roles:
                  | import("@solid-primitives/map").ReactiveMap<
                      string,
                      {
                        name: string;
                        permissions: {
                          a: number;
                          d: number;
                        };
                        colour?: string | null | undefined;
                        hoist?: boolean | undefined;
                        rank?: number | undefined;
                      }
                    >
                  | undefined;
                readonly defaultPermissions: number;
                readonly flags: import("./hydration/server").ServerFlags;
                readonly analytics: boolean;
                readonly discoverable: boolean;
                readonly mature: boolean;
                readonly orderedChannels: (Omit<
                  {
                    id: string;
                    title: string;
                    channels: string[];
                  },
                  "channels"
                > & {
                  channels: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly type:
                      | "SavedMessages"
                      | "DirectMessage"
                      | "Group"
                      | "TextChannel"
                      | "VoiceChannel";
                    readonly name: string;
                    readonly description: string | undefined;
                    readonly icon:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly active: boolean;
                    readonly recipients: {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }[];
                    readonly user:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly owner:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly server:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly permissions: number | undefined;
                    readonly defaultPermissions:
                      | {
                          a: number;
                          d: number;
                        }
                      | undefined;
                    readonly rolePermissions:
                      | Record<
                          string,
                          {
                            a: number;
                            d: number;
                          }
                        >
                      | undefined;
                    readonly mature: boolean;
                    readonly lastMessageId: string | undefined;
                  }[];
                })[];
                readonly defaultChannel:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly type:
                        | "SavedMessages"
                        | "DirectMessage"
                        | "Group"
                        | "TextChannel"
                        | "VoiceChannel";
                      readonly name: string;
                      readonly description: string | undefined;
                      readonly icon:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly active: boolean;
                      readonly recipients: {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }[];
                      readonly user:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly owner:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly server:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly permissions: number | undefined;
                      readonly defaultPermissions:
                        | {
                            a: number;
                            d: number;
                          }
                        | undefined;
                      readonly rolePermissions:
                        | Record<
                            string,
                            {
                              a: number;
                              d: number;
                            }
                          >
                        | undefined;
                      readonly mature: boolean;
                      readonly lastMessageId: string | undefined;
                    }
                  | undefined;
                readonly orderedRoles: {
                  name: string;
                  permissions: {
                    a: number;
                    d: number;
                  };
                  colour?: string | null | undefined;
                  hoist?: boolean | undefined;
                  rank?: number | undefined;
                  id: string;
                }[];
              }
            | undefined;
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly joinedAt: Date;
          readonly nickname: string | undefined;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly roles: string[] | undefined;
          readonly timeout: Date | undefined;
        }
      | undefined;
    fetch(id: { server: string; user: string }): Promise<
      | {
          readonly id: {
            server: string;
            user: string;
          };
          readonly server:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly owner:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }
                  | undefined;
                readonly name: string;
                readonly description: string | undefined;
                readonly icon:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly banner:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly channels: {
                  readonly id: string;
                  readonly createdAt: Date;
                  readonly type:
                    | "SavedMessages"
                    | "DirectMessage"
                    | "Group"
                    | "TextChannel"
                    | "VoiceChannel";
                  readonly name: string;
                  readonly description: string | undefined;
                  readonly icon:
                    | {
                        _id: string;
                        tag: string;
                        filename: string;
                        metadata:
                          | {
                              type: "File";
                            }
                          | {
                              type: "Text";
                            }
                          | {
                              type: "Image";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Video";
                              width: number;
                              height: number;
                            }
                          | {
                              type: "Audio";
                            };
                        content_type: string;
                        size: number;
                        deleted?: boolean | null | undefined;
                        reported?: boolean | null | undefined;
                        message_id?: string | null | undefined;
                        user_id?: string | null | undefined;
                        server_id?: string | null | undefined;
                        object_id?: string | null | undefined;
                      }
                    | undefined;
                  readonly active: boolean;
                  readonly recipients: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly username: string;
                    readonly avatar:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly badges: import("./hydration/user").UserBadges;
                    readonly status:
                      | {
                          text?: string | null | undefined;
                          presence?:
                            | "Online"
                            | "Idle"
                            | "Focus"
                            | "Busy"
                            | "Invisible"
                            | null
                            | undefined;
                        }
                      | undefined;
                    readonly relationship:
                      | "None"
                      | "User"
                      | "Friend"
                      | "Outgoing"
                      | "Incoming"
                      | "Blocked"
                      | "BlockedOther";
                    readonly online: boolean;
                    readonly privileged: boolean;
                    readonly flags: import("./hydration/user").UserFlags;
                    readonly bot:
                      | {
                          owner: string;
                        }
                      | undefined;
                  }[];
                  readonly user:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly owner:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly server:
                    | {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }
                    | undefined;
                  readonly permissions: number | undefined;
                  readonly defaultPermissions:
                    | {
                        a: number;
                        d: number;
                      }
                    | undefined;
                  readonly rolePermissions:
                    | Record<
                        string,
                        {
                          a: number;
                          d: number;
                        }
                      >
                    | undefined;
                  readonly mature: boolean;
                  readonly lastMessageId: string | undefined;
                }[];
                readonly categories:
                  | {
                      id: string;
                      title: string;
                      channels: string[];
                    }[]
                  | undefined;
                readonly systemMessages:
                  | {
                      user_joined?: string | null | undefined;
                      user_left?: string | null | undefined;
                      user_kicked?: string | null | undefined;
                      user_banned?: string | null | undefined;
                    }
                  | undefined;
                readonly roles:
                  | import("@solid-primitives/map").ReactiveMap<
                      string,
                      {
                        name: string;
                        permissions: {
                          a: number;
                          d: number;
                        };
                        colour?: string | null | undefined;
                        hoist?: boolean | undefined;
                        rank?: number | undefined;
                      }
                    >
                  | undefined;
                readonly defaultPermissions: number;
                readonly flags: import("./hydration/server").ServerFlags;
                readonly analytics: boolean;
                readonly discoverable: boolean;
                readonly mature: boolean;
                readonly orderedChannels: (Omit<
                  {
                    id: string;
                    title: string;
                    channels: string[];
                  },
                  "channels"
                > & {
                  channels: {
                    readonly id: string;
                    readonly createdAt: Date;
                    readonly type:
                      | "SavedMessages"
                      | "DirectMessage"
                      | "Group"
                      | "TextChannel"
                      | "VoiceChannel";
                    readonly name: string;
                    readonly description: string | undefined;
                    readonly icon:
                      | {
                          _id: string;
                          tag: string;
                          filename: string;
                          metadata:
                            | {
                                type: "File";
                              }
                            | {
                                type: "Text";
                              }
                            | {
                                type: "Image";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Video";
                                width: number;
                                height: number;
                              }
                            | {
                                type: "Audio";
                              };
                          content_type: string;
                          size: number;
                          deleted?: boolean | null | undefined;
                          reported?: boolean | null | undefined;
                          message_id?: string | null | undefined;
                          user_id?: string | null | undefined;
                          server_id?: string | null | undefined;
                          object_id?: string | null | undefined;
                        }
                      | undefined;
                    readonly active: boolean;
                    readonly recipients: {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly username: string;
                      readonly avatar:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly badges: import("./hydration/user").UserBadges;
                      readonly status:
                        | {
                            text?: string | null | undefined;
                            presence?:
                              | "Online"
                              | "Idle"
                              | "Focus"
                              | "Busy"
                              | "Invisible"
                              | null
                              | undefined;
                          }
                        | undefined;
                      readonly relationship:
                        | "None"
                        | "User"
                        | "Friend"
                        | "Outgoing"
                        | "Incoming"
                        | "Blocked"
                        | "BlockedOther";
                      readonly online: boolean;
                      readonly privileged: boolean;
                      readonly flags: import("./hydration/user").UserFlags;
                      readonly bot:
                        | {
                            owner: string;
                          }
                        | undefined;
                    }[];
                    readonly user:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly owner:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly server:
                      | {
                          readonly id: string;
                          readonly createdAt: Date;
                          readonly username: string;
                          readonly avatar:
                            | {
                                _id: string;
                                tag: string;
                                filename: string;
                                metadata:
                                  | {
                                      type: "File";
                                    }
                                  | {
                                      type: "Text";
                                    }
                                  | {
                                      type: "Image";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Video";
                                      width: number;
                                      height: number;
                                    }
                                  | {
                                      type: "Audio";
                                    };
                                content_type: string;
                                size: number;
                                deleted?: boolean | null | undefined;
                                reported?: boolean | null | undefined;
                                message_id?: string | null | undefined;
                                user_id?: string | null | undefined;
                                server_id?: string | null | undefined;
                                object_id?: string | null | undefined;
                              }
                            | undefined;
                          readonly badges: import("./hydration/user").UserBadges;
                          readonly status:
                            | {
                                text?: string | null | undefined;
                                presence?:
                                  | "Online"
                                  | "Idle"
                                  | "Focus"
                                  | "Busy"
                                  | "Invisible"
                                  | null
                                  | undefined;
                              }
                            | undefined;
                          readonly relationship:
                            | "None"
                            | "User"
                            | "Friend"
                            | "Outgoing"
                            | "Incoming"
                            | "Blocked"
                            | "BlockedOther";
                          readonly online: boolean;
                          readonly privileged: boolean;
                          readonly flags: import("./hydration/user").UserFlags;
                          readonly bot:
                            | {
                                owner: string;
                              }
                            | undefined;
                        }
                      | undefined;
                    readonly permissions: number | undefined;
                    readonly defaultPermissions:
                      | {
                          a: number;
                          d: number;
                        }
                      | undefined;
                    readonly rolePermissions:
                      | Record<
                          string,
                          {
                            a: number;
                            d: number;
                          }
                        >
                      | undefined;
                    readonly mature: boolean;
                    readonly lastMessageId: string | undefined;
                  }[];
                })[];
                readonly defaultChannel:
                  | {
                      readonly id: string;
                      readonly createdAt: Date;
                      readonly type:
                        | "SavedMessages"
                        | "DirectMessage"
                        | "Group"
                        | "TextChannel"
                        | "VoiceChannel";
                      readonly name: string;
                      readonly description: string | undefined;
                      readonly icon:
                        | {
                            _id: string;
                            tag: string;
                            filename: string;
                            metadata:
                              | {
                                  type: "File";
                                }
                              | {
                                  type: "Text";
                                }
                              | {
                                  type: "Image";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Video";
                                  width: number;
                                  height: number;
                                }
                              | {
                                  type: "Audio";
                                };
                            content_type: string;
                            size: number;
                            deleted?: boolean | null | undefined;
                            reported?: boolean | null | undefined;
                            message_id?: string | null | undefined;
                            user_id?: string | null | undefined;
                            server_id?: string | null | undefined;
                            object_id?: string | null | undefined;
                          }
                        | undefined;
                      readonly active: boolean;
                      readonly recipients: {
                        readonly id: string;
                        readonly createdAt: Date;
                        readonly username: string;
                        readonly avatar:
                          | {
                              _id: string;
                              tag: string;
                              filename: string;
                              metadata:
                                | {
                                    type: "File";
                                  }
                                | {
                                    type: "Text";
                                  }
                                | {
                                    type: "Image";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Video";
                                    width: number;
                                    height: number;
                                  }
                                | {
                                    type: "Audio";
                                  };
                              content_type: string;
                              size: number;
                              deleted?: boolean | null | undefined;
                              reported?: boolean | null | undefined;
                              message_id?: string | null | undefined;
                              user_id?: string | null | undefined;
                              server_id?: string | null | undefined;
                              object_id?: string | null | undefined;
                            }
                          | undefined;
                        readonly badges: import("./hydration/user").UserBadges;
                        readonly status:
                          | {
                              text?: string | null | undefined;
                              presence?:
                                | "Online"
                                | "Idle"
                                | "Focus"
                                | "Busy"
                                | "Invisible"
                                | null
                                | undefined;
                            }
                          | undefined;
                        readonly relationship:
                          | "None"
                          | "User"
                          | "Friend"
                          | "Outgoing"
                          | "Incoming"
                          | "Blocked"
                          | "BlockedOther";
                        readonly online: boolean;
                        readonly privileged: boolean;
                        readonly flags: import("./hydration/user").UserFlags;
                        readonly bot:
                          | {
                              owner: string;
                            }
                          | undefined;
                      }[];
                      readonly user:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly owner:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly server:
                        | {
                            readonly id: string;
                            readonly createdAt: Date;
                            readonly username: string;
                            readonly avatar:
                              | {
                                  _id: string;
                                  tag: string;
                                  filename: string;
                                  metadata:
                                    | {
                                        type: "File";
                                      }
                                    | {
                                        type: "Text";
                                      }
                                    | {
                                        type: "Image";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Video";
                                        width: number;
                                        height: number;
                                      }
                                    | {
                                        type: "Audio";
                                      };
                                  content_type: string;
                                  size: number;
                                  deleted?: boolean | null | undefined;
                                  reported?: boolean | null | undefined;
                                  message_id?: string | null | undefined;
                                  user_id?: string | null | undefined;
                                  server_id?: string | null | undefined;
                                  object_id?: string | null | undefined;
                                }
                              | undefined;
                            readonly badges: import("./hydration/user").UserBadges;
                            readonly status:
                              | {
                                  text?: string | null | undefined;
                                  presence?:
                                    | "Online"
                                    | "Idle"
                                    | "Focus"
                                    | "Busy"
                                    | "Invisible"
                                    | null
                                    | undefined;
                                }
                              | undefined;
                            readonly relationship:
                              | "None"
                              | "User"
                              | "Friend"
                              | "Outgoing"
                              | "Incoming"
                              | "Blocked"
                              | "BlockedOther";
                            readonly online: boolean;
                            readonly privileged: boolean;
                            readonly flags: import("./hydration/user").UserFlags;
                            readonly bot:
                              | {
                                  owner: string;
                                }
                              | undefined;
                          }
                        | undefined;
                      readonly permissions: number | undefined;
                      readonly defaultPermissions:
                        | {
                            a: number;
                            d: number;
                          }
                        | undefined;
                      readonly rolePermissions:
                        | Record<
                            string,
                            {
                              a: number;
                              d: number;
                            }
                          >
                        | undefined;
                      readonly mature: boolean;
                      readonly lastMessageId: string | undefined;
                    }
                  | undefined;
                readonly orderedRoles: {
                  name: string;
                  permissions: {
                    a: number;
                    d: number;
                  };
                  colour?: string | null | undefined;
                  hoist?: boolean | undefined;
                  rank?: number | undefined;
                  id: string;
                }[];
              }
            | undefined;
          readonly user:
            | {
                readonly id: string;
                readonly createdAt: Date;
                readonly username: string;
                readonly avatar:
                  | {
                      _id: string;
                      tag: string;
                      filename: string;
                      metadata:
                        | {
                            type: "File";
                          }
                        | {
                            type: "Text";
                          }
                        | {
                            type: "Image";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Video";
                            width: number;
                            height: number;
                          }
                        | {
                            type: "Audio";
                          };
                      content_type: string;
                      size: number;
                      deleted?: boolean | null | undefined;
                      reported?: boolean | null | undefined;
                      message_id?: string | null | undefined;
                      user_id?: string | null | undefined;
                      server_id?: string | null | undefined;
                      object_id?: string | null | undefined;
                    }
                  | undefined;
                readonly badges: import("./hydration/user").UserBadges;
                readonly status:
                  | {
                      text?: string | null | undefined;
                      presence?:
                        | "Online"
                        | "Idle"
                        | "Focus"
                        | "Busy"
                        | "Invisible"
                        | null
                        | undefined;
                    }
                  | undefined;
                readonly relationship:
                  | "None"
                  | "User"
                  | "Friend"
                  | "Outgoing"
                  | "Incoming"
                  | "Blocked"
                  | "BlockedOther";
                readonly online: boolean;
                readonly privileged: boolean;
                readonly flags: import("./hydration/user").UserFlags;
                readonly bot:
                  | {
                      owner: string;
                    }
                  | undefined;
              }
            | undefined;
          readonly joinedAt: Date;
          readonly nickname: string | undefined;
          readonly avatar:
            | {
                _id: string;
                tag: string;
                filename: string;
                metadata:
                  | {
                      type: "File";
                    }
                  | {
                      type: "Text";
                    }
                  | {
                      type: "Image";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Video";
                      width: number;
                      height: number;
                    }
                  | {
                      type: "Audio";
                    };
                content_type: string;
                size: number;
                deleted?: boolean | null | undefined;
                reported?: boolean | null | undefined;
                message_id?: string | null | undefined;
                user_id?: string | null | undefined;
                server_id?: string | null | undefined;
                object_id?: string | null | undefined;
              }
            | undefined;
          readonly roles: string[] | undefined;
          readonly timeout: Date | undefined;
        }
      | undefined
    >;
  };
  readonly api: API;
  readonly baseURL: string;
  readonly events: EventClient<1>;
  configuration: RevoltConfig | undefined;
  session: Session | undefined;
  /**
   * Create Revolt.js Client
   */
  constructor(baseURL: string);
  /**
   * Connect
   */
  connect(): void;
  /**
   * Log in with auth data, creating a new session in the process.
   * @param details Login data object
   * @returns An on-boarding function if on-boarding is required, undefined otherwise
   */
  login(details: DataLogin): Promise<void>;
  /**
   * Use an existing session to log into Revolt.
   * @param session Session data object
   * @returns An on-boarding function if on-boarding is required, undefined otherwise
   */
  useExistingSession(session: Session): Promise<void>;
  /**
   * Log in as a bot.
   * @param token Bot token
   */
  loginBot(token: string): Promise<void>;
}
export {};
//# sourceMappingURL=Client.d.ts.map
